<?php
namespace TYPO3\ExlPageAdvanced\Utility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Romain CANON <romain.canon@exl-group.com>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use \TYPO3\CMS\Core\Utility\GeneralUtility;
use \TYPO3\CMS\Frontend\Utility\EidUtility;

/**
 * This functions allows you to easily call an action in a controller with
 * an ajax call.
 * Inspired of : http://t3-developer.com/extbase-fluid/cheats-extbase/extbase-allgemein/ajax-dispatcher-eid-in-typo3-61/
 *
 * @package TYPO3
 * @subpackage tx_sitefactory
 */
class AjaxDispatcher {
	/** @var $objectManager \TYPO3\CMS\Extbase\Object\ObjectManager */
	private $objectManager;

	/** @var $params array */
	private $params;

	public function run() {
		global $TYPO3_CONF_VARS;

		// Gets the Ajax call parameters.
		$this->params = GeneralUtility::_GP('request');

		if ($this->params['bootstrap']) {
			self::setupBootstrap();
		}

		if (isset($this->params['mvc'])) {
			$this->objectManager = GeneralUtility::makeInstance('\\TYPO3\\CMS\\Extbase\\Object\\ObjectManager');

			$mvcArguments = $this->params['mvc'];

			$GLOBALS['TT'] = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('\\TYPO3\\CMS\\Core\\TimeTracker\\TimeTracker');
			$GLOBALS['TT']->start();

			if(TYPO3_MODE == 'FE') {
				/** @var $GLOBALS \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController */
				$GLOBALS['TSFE'] = GeneralUtility::makeInstance('\\TYPO3\\CMS\\Frontend\\Controller\\TypoScriptFrontendController', $TYPO3_CONF_VARS, $this->params['id'], 0);
				EidUtility::initLanguage();
				EidUtility::initTCA();

				// Important: no Cache for Ajax stuff
				$GLOBALS['TSFE']->set_no_cache();

				$GLOBALS['TSFE']->initFEuser();
//				$GLOBALS['TSFE']->checkAlternativeIdMethods();
				$GLOBALS['TSFE']->determineId();
				$GLOBALS['TSFE']->initTemplate();
				$GLOBALS['TSFE']->getPageAndRootline();
				$GLOBALS['TSFE']->getConfigArray();
				$GLOBALS['TSFE']->connectToDB();
				$GLOBALS['TSFE']->settingLanguage();
			}

			/** @var $request \TYPO3\CMS\Extbase\Mvc\Request */
			$request = $this->objectManager->get('\\TYPO3\\CMS\\Extbase\\Mvc\\RequestInterface');

			// Build the request.
			$request->setControllerVendorName($mvcArguments['vendor']);
			$request->setcontrollerExtensionName($mvcArguments['extensionName']);
			$request->setPluginName($mvcArguments['pluginName']);
			$request->setControllerName($mvcArguments['controller']);
			$request->setControllerActionName($mvcArguments['action']);
			$request->setFormat('html');
			$request->setArguments($this->params['arguments']);

			/** @var $response \TYPO3\CMS\Extbase\Mvc\ResponseInterface */
			$response = $this->objectManager->get('\\TYPO3\\CMS\\Extbase\\Mvc\\ResponseInterface');

			/** @var $dispatcher \TYPO3\CMS\Extbase\Mvc\Dispatcher */
			$dispatcher = $this->objectManager->get('\\TYPO3\\CMS\\Extbase\\Mvc\\Dispatcher');
			$dispatcher->dispatch($request, $response);

			$result = $response->getContent();
		}
		else {
			$result = GeneralUtility::callUserFunction($this->params['function'], $this->params['arguments'], $this);
		}

		echo $result;
	}

	public static  function setupBootstrap() {
		// Define the cliMode so we can omit to set the pluginName in the request.
		define('TYPO3_cliMode', true);

		/** @var $bootstrap \TYPO3\CMS\Extbase\Core\Bootstrap */
		$bootstrap = GeneralUtility::makeInstance('\\TYPO3\\CMS\\Extbase\\Core\\Bootstrap');

		$bootstrapConf = array();
		$bootstrap->initialize($bootstrapConf);
	}
}

// When called from Frontend, TYPO3 doesn't allow to define the ajaxDispatcher "run" function, so we need to call
// it ourself.
if(TYPO3_MODE == 'FE') {
	/** @var $ajaxDispatcher \TYPO3\ExlUtilities\Utility\AjaxDispatcher*/
	$ajaxDispatcher = new AjaxDispatcher();
	$ajaxDispatcher->run();
}